/*
 * FILE: callreg.c
 * 
 * ABSTRACT: Contains functions to call the external Perl script reg.2.1.4.
 * 
 * Parts of this file is cut & past from the manual page perlembed.
 * 
 * $Id: callreg.c,v 1.1.1.1 1998/12/30 18:40:32 rks Exp $
 * 
 * CHANGE LOG:
 * ##  who  ddMmmyy  defect  description
 * ==  ===  =======  ======  =================================================
 *  1  rks  18Oct98  --      now closing file in perlcommand _> no zombies
 *  0  rks  28Jun98  -       initial
 */ 

#define DBG_REG 0

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include <forms.h>

#include "dbgmem.h"

/*
 * bool is nowhere defined on my Linux system, so
 * I add it here, because perl needs it
 */
typedef char bool;

#include <EXTERN.h>
#include <perl.h>

#include "output.h"
#include "globals.h"
#include "tools.h"


extern char            cfHeiseFile[];
extern char            cfRegPerlScript[];

static PerlInterpreter *my_perl;

char perlcommand[]=
"$state=0;"
"$foundcount=-1;"
"open(reg_out, \"%s -f %s %s \\\"%s\\\" |\");"
"while(<reg_out>)"
"{"
"    if(/^Done. Found \\d+ match.$/)"
"    {"
"        $state=0;"
"    }"
"    if( $state == 1)"
"    {"
"        if(/^=======+$/)"
"        {"
"            $foundcount++,"
"        }"
"        else"
"        {"
"            if(/^Titel       : (.*$)/)"
"            {"
"                @show_list[$foundcount]=\"$1\";"
"            }"
"            elsif(/^Zeitschrift : (.*$)/)"
"            {"
"                @show_list[$foundcount]=\"@show_list[$foundcount] ($1)\";"
"            }"
"            "
"            @full_list[$foundcount]=\"@full_list[$foundcount]$_\";"
"        }"
"        "
"    }"
"    if(/^Searching for: /)"
"    {"
"        $state=1;"
"    }"
"}"
"close(reg_out);"
"return 0;"
;


int perl_init(void)
{
    char        *embedding[] = { "", "-e", "sub _eval_ { eval $_[0] }", NULL};
    I32          rc;

    my_perl = perl_alloc();
    if(my_perl == NULL)
    {
        fprintf(stderr, "my_perl = NULL");
        exit(1);
    }

#if DBG_REG
    fprintf(stderr, "Search for: \"%s\"\n", argv_buffer);
#endif

    perl_construct( my_perl );
    
    /*
     * perl_destruct_level = ;
     */

    rc = perl_parse(my_perl, NULL, 3, embedding, NULL);
#if DBG_REG
    fprintf(stderr, "perl_parse rc: %ld\n", rc);
#endif
    
    return rc;
}

int perl_eval(char *string)
{
    char *argv[2];
    argv[0] = string;
    argv[1] = NULL;
    return perl_call_argv("_eval_", 0, argv);
}                                                              


/*
 * FUNCTION: perl_reg
 * 
 * ABSTRACT:
 * Searches through Heise register by calling external Perl script
 * REG and feeds the output into the variables matches and fulltext.
 * 
 * PARAMETER:
 * see code
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int perl_reg(char *argv_buffer, char *parameter, char **matches[],
        char **fulltext[])
{
    int          i;
    SV          *current_match;
    AV          *show_list;
    AV          *full_list;
    I32          num_matches;
    I32          rc;
    STRLEN       length;
    char         command_buffer[4096];
    

    sprintf(command_buffer, perlcommand, cfRegPerlScript, cfHeiseFile,
            parameter, argv_buffer);


    if(my_perl == NULL)
    {
        perl_init();
    }


    rc = perl_eval(command_buffer);
    
#if DBG_REG
    fprintf(stderr, "perl_eval rc: %ld\n", rc);
#endif

    show_list = perl_get_av("show_list", FALSE);
    full_list = perl_get_av("full_list", FALSE);
    num_matches = av_len(show_list) + 1; /** assume $[ is 0 **/
    (*matches) = (char **) Malloc(sizeof(char *) * (num_matches + 1));
    (*fulltext) = (char **) Malloc(sizeof(char *) * (num_matches + 1));
    for (i = num_matches-1; i >= 0; i-- )
    {
        char    *ptr;
        if(((num_matches-i) % 25) == 0)
        {
            status("Getting %d articles from perl: %d", num_matches,
                    num_matches - i);
        }
        current_match = av_shift(show_list);
        ptr = SvPV(current_match, length);
        (*matches)[i] = Malloc(length+1);
        strcpy((*matches)[i], ptr);
        SvREFCNT_dec(current_match);

        current_match = av_shift(full_list);
        ptr = SvPV(current_match, length);
        (*fulltext)[i] = Malloc(length+1);
        strcpy((*fulltext)[i], ptr);
        SvREFCNT_dec(current_match);
#if DBG_REG
        printf("%s\n%s\n\n", (*matches)[i], (*fulltext)[i]);
#endif
    }
    (*matches)[num_matches]  = NULL;
    (*fulltext)[num_matches] = NULL;
    
    av_undef(show_list);
    av_undef(full_list);

    return num_matches;
}


char *check_command =
"$update=\"%s\";"
"$errorline=-1;"
"%%journal = ('c'=>'c\\'t', 'i'=>'iX', 'g'=>'Gateway', 't'=>\"Test\", 'f'=>'Finanztest' );"
"open (UPD, $update) or die \"cannot open $update: $!\\n\";"
"$journals = join '', keys %%journal;"
"$journals = uc ($journals) . lc ($journals);"
"$line = 0;"
"while(<UPD>) {"
"    last if $line == 4 && ! /^\\s*\\d+\\r?$/;"
"    last if $line == 5 && ! /^[\\s012]\\d\\r?$/;"
"    last if $line == 6 && ! /^[$journals]\\d\\d(?:\\d\\d)?\\r?$/o;"
"    $line = 0 if ++$line == 9;"
"}"
"if ($line != 0)"
"{"
"    $errorline = $.;"
"}"
"else"
"{"
"    $errorline = 0;"
"}"
;
                                                                    

/*
 * FUNCTION: check_input
 * 
 * ABSTRACT:
 * Validates a Heise register file. I've stolen some perl code from
 * reg.2.1.4 to do this.
 * 
 * PARAMETER:
 * see code
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int check_input(char *filename)
{
    char command_buffer[4096];
    long errorline;
    int  rc;


    sprintf(command_buffer, check_command, filename); 
#if DBG_REG
    printf("%s\n", command_buffer);
#endif

    if(my_perl == NULL)
    {
        perl_init();
    }

    rc = perl_eval(command_buffer);
    
    errorline = SvIV(perl_get_sv("errorline", FALSE));

    if(errorline != 0)
    {
        sprintf(command_buffer, "There is something wrong around line %ld.",
                errorline);
        fl_show_message("Heise Register File invalid!", command_buffer,
                "Use editor to correct the file");
        return 1;
    }
    else
        fl_show_message("OK!", "Heise Register File is valid.", "");
    
    return 0;
}


/*
 * FUNCTION: add_update
 * 
 * ABSTRACT:
 * Adds the given Update file (*.cti) to the Heise register file using 
 * reg.
 * Output is written to a special window (defined in output.c).
 * 
 * PARAMETER:
 * see code
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int add_update(char *file)
{
    char buffer[2048];
    char               *ptr;
    FD_update_output *fd_output;
    FILE        *fd;
    
    fd_output = create_form_update_output();
    
    fl_show_form(fd_output->update_output, FL_PLACE_CENTER, FL_TRANSIENT,
                        "Adding File...");
    
    fl_deactivate_object(fd_output->button);
    fl_set_object_lcolor(fd_output->button,FL_INACTIVE_COL);

    status(buffer, "Spawning %s ...", cfRegPerlScript);
    sprintf(buffer, "Spawning %s ...\n", cfRegPerlScript);
    fl_add_browser_line(fd_output->browser, buffer);
    
#define CHOICE_TO_PARAM(i) ((i)==YES)?"on":((i)==NO)?"off":"ERROR"

    sprintf(buffer, "%s -check %s -dup %s -f %s -u %s 2>&1",
            cfRegPerlScript,
            CHOICE_TO_PARAM(cfCheck), CHOICE_TO_PARAM(cfDupes),
             cfHeiseFile, file);
    
    
#if DBG_REG
    printf("%s\n",buffer);
#endif

    fd = popen(buffer, "r");
    
    ptr = buffer;
    while(fread(ptr, 1, 1, fd) == 1)
    {
        *(ptr+1) = 0;
        if(*ptr == '\n' || *ptr == '.' || *ptr == 0)
        {
            fl_addto_browser_chars(fd_output->browser, buffer);
            ptr = buffer;
        }
        else
        {
            ptr++;
        }
    }
    pclose(fd);
    
    fl_set_object_lcolor(fd_output->button,FL_BLACK);
    fl_activate_object(fd_output->button);

    fl_do_forms();
    
    fl_hide_form(fd_output->update_output);
    fl_free_form(fd_output->update_output);
    fl_activate_form(fd_main->main);

    
    return 0;

}

/*--------[ END OF FILE callreg.c ]------------------------------------------*/
