/*
 * FILE: config.c
 * 
 * ABSTRACT: configuration window form
 * 
 * Contains form definition for the configuration window emitted by fdesign and
 * hand edited afterwards and function to read / write the configuration file.
 * Here are also functions to validate the configuration.
 * 
 * $Id: config.c,v 1.2 1998/12/30 19:07:13 rks Exp $
 * 
 * CHANGE LOG:
 * ##  who  ddMmmyy  defect  description
 * ==  ===  =======  ======  =================================================
 *  0  rks  28Jun98  -       initial
 */ 

#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include "forms.h"

#include "config.h"
#include "globals.h"
#include "tools.h"

FD_config *fd_config;

/*
 * FUNCTION: create_form_config
 * 
 * ABSTRACT:
 * Creates configuration window. Designed with fdesign, edited afterwards.
 * 
 * PARAMETER:
 * None
 * 
 * RETURNVALUE:
 * Pointer to created form
 * 
 */ 
FD_config *create_form_config(void)
{
  FL_OBJECT *obj;
  FD_config *fdui = (FD_config *) fl_calloc(1, sizeof(*fdui));
  int        dy;

  fdui->config = fl_bgn_form(FL_NO_BOX, 410, 580);
  obj = fl_add_box(FL_UP_BOX,0,0,410,580,"");
    fl_set_object_resize(obj, FL_RESIZE_NONE);

  /* font size */
  obj = fl_add_labelframe(FL_ENGRAVED_FRAME,20,50,370,50,"Fontsize");

  /* small, normal, medium, large button */
  obj = fl_add_lightbutton(FL_RADIO_BUTTON,30,60,80,30,"Small");
    fl_set_object_lsize(obj,FL_SMALL_SIZE);
    fl_set_object_callback(obj,cfSetFontSize,FL_SMALL_SIZE);
    if(cfNewFontSize == FL_SMALL_SIZE) fl_set_button(obj, 1);
  obj = fl_add_lightbutton(FL_RADIO_BUTTON,120,60,80,30,"Normal");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_callback(obj,cfSetFontSize,FL_NORMAL_SIZE);
    if(cfNewFontSize == FL_NORMAL_SIZE) fl_set_button(obj, 1);
  obj = fl_add_lightbutton(FL_RADIO_BUTTON,210,60,80,30,"Medium");
    fl_set_object_lsize(obj,FL_MEDIUM_SIZE);
    fl_set_object_callback(obj,cfSetFontSize,FL_MEDIUM_SIZE);
    if(cfNewFontSize == FL_MEDIUM_SIZE) fl_set_button(obj, 1);
  obj = fl_add_lightbutton(FL_RADIO_BUTTON,300,60,80,30,"Large");
    fl_set_object_lsize(obj,FL_LARGE_SIZE);
    fl_set_object_callback(obj,cfSetFontSize,FL_LARGE_SIZE);
    if(cfNewFontSize == FL_LARGE_SIZE) fl_set_button(obj, 1);

  /* input file */
  obj = fl_add_labelframe(FL_ENGRAVED_FRAME,20,120,370,90,
          "Heise Register File");

  fdui->cfInputFile = obj = fl_add_input(FL_NORMAL_INPUT,30,130,350,30,"");
    fl_set_input(obj, cfNewHeiseFile);

  obj = fl_add_button(FL_NORMAL_BUTTON,30,170,80,30,"Browse");
    fl_set_object_callback(obj,cfBrowseInputFile,0);
  obj = fl_add_button(FL_NORMAL_BUTTON,300,170,80,30,"Check");
    fl_set_object_callback(obj,cfCheckInputFile,0);

  /* perl script */
  obj = fl_add_labelframe(FL_ENGRAVED_FRAME,20,230,370,90,"REG Perl Script");
  fdui->cfRegScript = obj = fl_add_input(FL_NORMAL_INPUT,30,240,350,30,"");
    fl_set_input(obj, cfNewRegPerlScript);

  obj = fl_add_button(FL_NORMAL_BUTTON,30,280,80,30,"Browse");
    fl_set_object_callback(obj,cfBrowseRegScript,0);

  /* SEARCH defaults */
  dy = 340;
  obj = fl_add_labelframe(FL_ENGRAVED_FRAME,20,dy,240,150,
          "REG Search Defaults");

  obj = fl_add_choice(FL_NORMAL_CHOICE2,30,dy+40,90,30,"Umlaut Convert");
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    fl_set_object_callback(obj,cfCbUmlaut,0);
    fl_addto_choice(obj, "Yes|No|Auto");
   fl_set_choice(obj,cfUmlaut);

  obj = fl_add_choice(FL_NORMAL_CHOICE2,150,dy+40,90,30,"Case Sensetive");
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    fl_set_object_callback(obj,cfCbCase,0);
    fl_addto_choice(obj, "Yes|No|Auto");
    fl_set_choice(obj,cfCase);

  obj = fl_add_choice(FL_NORMAL_CHOICE2,30,dy+100,90,30,"Regular Expr");
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    fl_set_object_callback(obj,cfCbRegExpr,0);
    fl_addto_choice(obj, "Yes|No|Auto");
    fl_set_choice(obj,cfRegExpr);

  obj = fl_add_choice(FL_NORMAL_CHOICE2,150,dy+100,90,30,"Phonetic");
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    fl_set_object_callback(obj,cfCbPhonetic,0);
    fl_addto_choice(obj, "Yes|No");
    fl_set_choice(obj,cfPhonetic);


  obj = fl_add_labelframe(FL_ENGRAVED_FRAME,260,dy,130,150,"REG Update");

  obj = fl_add_choice(FL_NORMAL_CHOICE2,280,dy+40,90,30,"Check Integrity");
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    fl_set_object_callback(obj,cfCbCheck,0);
    fl_addto_choice(obj, "Yes|No");
    fl_set_choice(obj,cfCheck);

  obj = fl_add_choice(FL_NORMAL_CHOICE2,280,dy+100,90,30,"Delete Dupes");
    fl_set_object_lalign(obj,FL_ALIGN_TOP_LEFT);
    fl_set_object_callback(obj,cfCbDupes,0);
    fl_addto_choice(obj, "Yes|No");
    fl_set_choice(obj,cfDupes);
    

  /* Apply / Save / Cancel Buttons */
  fdui->cfApplyButton = obj = fl_add_button(FL_NORMAL_BUTTON,30,510,80,30,
          "Apply");

  obj = fl_add_button(FL_NORMAL_BUTTON,160,510,80,30,"Save");
    fl_set_object_callback(obj,cfSave,0);

  fdui->cfCancelButton = obj = fl_add_button(FL_NORMAL_BUTTON,290,510,80,30,
          "Cancel");

  fl_end_form();

  fdui->config->fdui = fdui;

  return fdui;
}


#define SKIP_WS(x) while(isspace(*(x)) && (*(x) != 0)) (x)++;
#define NEXT_WS(x) while(!isspace(*(x)) && (*(x) != 0)) (x)++;
#define CONF_ERR                                                 \
                {                                                \
                    int col = walker - buffer+1;                 \
                    char  b[64];                                 \
                    sprintf(b, "Line %d, Col %d:", line, col);   \
                    fl_show_alert("Invalid config file!",        \
                              b, buffer, 0);                     \
                    fclose(fd);                                  \
                    return 2;                                    \
                }

/*
 * FUNCTION: read_conf
 * 
 * ABSTRACT:
 * Reads the configuration file.
 * 
 * PARAMETER:
 * None
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int read_conf(void)
{
    char        buffer[1024];
    char       *home_dir;
    char       *walker;
    char       *w2;
    int         font_done = 0;
    int         input_done = 0;
    int         reg_done = 0;
    FILE       *fd;
    int         line = 0;
    
    home_dir = getenv("HOME");
    if(home_dir == NULL)
    {
        fl_show_alert("ERROR!", "Can't determine your home directory.", 
                "Set environment variable 'HOME'!", 0);
    }
    strcpy(buffer, home_dir);
    strcat(buffer, "/");
    strcat(buffer, CFG_FILE_NAME);

    fd = fopen(buffer, "rt");
    if(fd == NULL) return 2;
    
    while(fgets(buffer, 1024, fd) != NULL)
    {
        line++;
        walker = buffer;
        SKIP_WS(walker);

        if(walker[0] == '#') continue;
        if(walker[0] == 0) continue;
        
        if(strncmp(walker, CFG_FONT, CFG_FONT_SIZE) == 0)
        {
            NEXT_WS(walker);
            if(!isspace(*walker)) CONF_ERR;
            SKIP_WS(walker);
            w2 = walker;
            NEXT_WS(w2);
            *w2 = 0;
            if(!strcmp(walker, CFG_FONT_SMALL))
            {
                cfFontSize = FL_SMALL_SIZE;
                font_done = 1;
            }
            else if(!strcmp(walker, CFG_FONT_NORMAL))
            {
                cfFontSize = FL_NORMAL_SIZE;
                font_done = 1;
            }
            else if(!strcmp(walker, CFG_FONT_MEDIUM))
            {
                cfFontSize = FL_MEDIUM_SIZE;
                font_done = 1;
            }
            else if(!strcmp(walker, CFG_FONT_LARGE))
            {
                cfFontSize = FL_LARGE_SIZE;
                font_done = 1;
                break;
            }
            else
            {
                CONF_ERR;
            }
        }
        else if(strncmp(walker, CFG_INPUT, CFG_INPUT_SIZE) == 0)
        {
            NEXT_WS(walker);
            if(!isspace(*walker)) CONF_ERR;
            SKIP_WS(walker);
            strcpy(cfHeiseFile, walker);
            walker = strchr(cfHeiseFile, '\n');
            if(walker) *walker = 0;
            input_done = 1;
        }
        else if(strncmp(walker, CFG_REG, CFG_REG_SIZE) == 0)
        {
            NEXT_WS(walker);
            if(!isspace(*walker)) CONF_ERR;
            SKIP_WS(walker);
            strcpy(cfRegPerlScript, walker);
            walker = strchr(cfRegPerlScript, '\n');
            if(walker) *walker = 0;
            reg_done = 1;
        }
        else if(strncmp(walker, CFG_UMLAUT, CFG_UMLAUT_SIZE) == 0)
        {
            NEXT_WS(walker);
            if(!isspace(*walker)) CONF_ERR;
            SKIP_WS(walker);
            w2 = walker;
            NEXT_WS(w2);
            *w2=0;
            if(!strcmp(walker, CFG_YES))        cfUmlaut = YES;
            else if(!strcmp(walker, CFG_NO))    cfUmlaut = NO;
            else if(!strcmp(walker, CFG_AUTO))  cfUmlaut = AUTO;
            else
                CONF_ERR;
        }
        else if(strncmp(walker, CFG_PHONETIC, CFG_PHONETIC_SIZE) == 0)
        {
            NEXT_WS(walker);
            if(!isspace(*walker)) CONF_ERR;
            SKIP_WS(walker);
            w2 = walker;
            NEXT_WS(w2);
            *w2=0;
            if(!strcmp(walker, CFG_YES))       cfPhonetic = YES;
            else if(!strcmp(walker, CFG_NO))  cfPhonetic = NO;
            else
                CONF_ERR;
        }
        else if(strncmp(walker, CFG_CASE, CFG_CASE_SIZE) == 0)
        {
            NEXT_WS(walker);
            if(!isspace(*walker)) CONF_ERR;
            SKIP_WS(walker);
            w2 = walker;
            NEXT_WS(w2);
            *w2=0;
            if(!strcmp(walker, CFG_YES))        cfCase = YES;
            else if(!strcmp(walker, CFG_NO))    cfCase = NO;
            else if(!strcmp(walker, CFG_AUTO))  cfCase = AUTO;
            else
                CONF_ERR;
        }
        else if(strncmp(walker, CFG_REGEXPR, CFG_REGEXPR_SIZE) == 0)
        {
            NEXT_WS(walker);
            if(!isspace(*walker)) CONF_ERR;
            SKIP_WS(walker);
            w2 = walker;
            NEXT_WS(w2);
            *w2=0;
            if(!strcmp(walker, CFG_YES))        cfRegExpr = YES;
            else if(!strcmp(walker, CFG_NO))    cfRegExpr = NO;
            else if(!strcmp(walker, CFG_AUTO))  cfRegExpr = AUTO;
            else
                CONF_ERR;
        }
        else if(strncmp(walker, CFG_CHECK, CFG_CHECK_SIZE) == 0)
        {
            NEXT_WS(walker);
            if(!isspace(*walker)) CONF_ERR;
            SKIP_WS(walker);
            w2 = walker;
            NEXT_WS(w2);
            *w2=0;
            if(!strcmp(walker, CFG_YES))        cfCheck = YES;
            else if(!strcmp(walker, CFG_NO))    cfCheck = NO;
            else
                CONF_ERR;
        }
        else if(strncmp(walker, CFG_DUPES, CFG_DUPES_SIZE) == 0)
        {
            NEXT_WS(walker);
            if(!isspace(*walker)) CONF_ERR;
            SKIP_WS(walker);
            w2 = walker;
            NEXT_WS(w2);
            *w2=0;
            if(!strcmp(walker, CFG_YES))        cfDupes = YES;
            else if(!strcmp(walker, CFG_NO))    cfDupes = NO;
            else
                CONF_ERR;
        }
        else
        {
            CONF_ERR;
            fl_show_alert("Invalid config file!", "Can't understand:", 
                    buffer, 0);
            fclose(fd);
            return 2;
        }
    }

    if(!font_done)
    {
            fl_show_alert("Invalid config file!", "No font size set.", "", 0);
            fclose(fd);
            return 2;
    }
    if(!input_done)
    {
            fl_show_alert("Invalid config file!", "No input file set.", "", 0);
            fclose(fd);
            return 2;
    }
    if(!reg_done)
    {
            fl_show_alert("Invalid config file!", "No reg Perl script set.", 
                    "", 0);
            fclose(fd);
            return 2;
    }
    fclose(fd);
    
    if(verify_conf())
    {
        return 2;
    }

    return 0;
}

#define CONF_INFO \
"#\n" \
"# Config file for xreg\n" \
"#\n" 

/*
 * FUNCTION: write_conf
 * 
 * ABSTRACT:
 * Writes the configuration file
 * 
 * PARAMETER:
 * None
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int write_conf(void)
{
    char        buffer[1024];
    char       *home_dir;
    FILE       *fd;
    
    home_dir = getenv("HOME");
    if(home_dir == NULL)
    {
        fl_show_alert("ERROR!", "Can't determine your home directory.", 
                "Set environment variable 'HOME'!", 0);
        exit(1);
    }
    strcpy(buffer, home_dir);
    strcat(buffer, "/");
    strcat(buffer, CFG_FILE_NAME);

    fd = fopen(buffer, "wt");
    if(fd == NULL) return 2;
    
    fputs(CONF_INFO, fd);
    fputs(CFG_FONT "\t", fd);
    switch(cfNewFontSize)
    {
      case FL_SMALL_SIZE:
        fputs(CFG_FONT_SMALL, fd);
        break;
      case FL_NORMAL_SIZE:
        fputs(CFG_FONT_NORMAL, fd);
        break;
      case FL_MEDIUM_SIZE:
        fputs(CFG_FONT_MEDIUM, fd);
        break;
      case FL_LARGE_SIZE:
        fputs(CFG_FONT_MEDIUM, fd);
        break;
      default:
        fputs(CFG_FONT_MEDIUM, fd);
        break;
    }
    fputs("\n", fd);
    
    strcpy(cfNewHeiseFile, fl_get_input(fd_config->cfInputFile));
    strcpy(cfNewRegPerlScript, fl_get_input(fd_config->cfRegScript));

    fprintf(fd, "input\t%s\n", cfNewHeiseFile);
    fprintf(fd, "reg\t%s\n", cfNewRegPerlScript);
    
    fprintf(fd, "\n# REG default parameter\n");

#define CHOICE_STR(i) ((i)==YES)?CFG_YES:((i)==NO)?CFG_NO: \
    ((i)==AUTO)?CFG_AUTO:"ERROR"

    fprintf(fd, CFG_UMLAUT "\t%s\n", CHOICE_STR(cfNewUmlaut));
    fprintf(fd, CFG_PHONETIC "\t%s\n", CHOICE_STR(cfNewPhonetic));
    fprintf(fd, CFG_CASE "\t%s\n", CHOICE_STR(cfNewCase));
    fprintf(fd, CFG_REGEXPR "\t%s\n", CHOICE_STR(cfNewRegExpr));
    fprintf(fd, CFG_CHECK "\t%s\n", CHOICE_STR(cfNewCheck));
    fprintf(fd, CFG_DUPES "\t%s\n", CHOICE_STR(cfNewDupes));
    
    fclose(fd);

    return 0;
}

/*
 * FUNCTION: verify_conf
 * 
 * ABSTRACT:
 * Verifies the configuration read in from the configuration file.
 * Only font size, Heise file and reg script is checked.
 * 
 * PARAMETER:
 * NONE
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int verify_conf(void)
{
    int                 rc;

    /* check if input file is readable */
    rc = access(cfHeiseFile, R_OK);
    if(rc != 0)
    {
        fl_show_alert("ERROR: Can't find Heise register file.", cfHeiseFile, 
                strerror(errno), 0);
        return 1;
    }

    rc = access(cfRegPerlScript, X_OK);
    if(rc != 0)
    {
        fl_show_alert("ERROR: reg Perl script not executable.", cfRegPerlScript, 
                strerror(errno), 0);
        return 1;
    }

    switch(cfFontSize)
    {
      case FL_SMALL_SIZE:
      case FL_NORMAL_SIZE:
      case FL_MEDIUM_SIZE:
      case FL_LARGE_SIZE:
        break;
      default:
        fl_show_alert("ERROR!", "Invalid font size.", 
                "", 0);
        return 1;
        break;

    }
    
    return 0;
}


/*
 * FUNCTION: verify_new_conf
 * 
 * ABSTRACT:
 * Verifies the configuration set by the user in the configuration window.
 * Only font size, Heise file and reg script is checked.
 * 
 * PARAMETER:
 * NONE
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int verify_new_conf(void)
{
    int                 rc;

    /* check if input file is readable */
    rc = access(cfNewHeiseFile, R_OK);
    if(rc != 0)
    {
        fl_show_alert("ERROR: Can't find Heise register file.", cfNewHeiseFile, 
                strerror(errno), 0);
        return 1;
    }

    rc = access(cfNewRegPerlScript, X_OK);
    if(rc != 0)
    {
        fl_show_alert("ERROR: reg Perl script not executable.",
                cfNewRegPerlScript, strerror(errno), 0);
        return 1;
    }
    
    switch(cfFontSize)
    {
      case FL_SMALL_SIZE:
      case FL_NORMAL_SIZE:
      case FL_MEDIUM_SIZE:
      case FL_LARGE_SIZE:
        break;
      default:
        fl_show_alert("ERROR!", "Invalid font size.", 
                "", 0);
        return 1;
        break;

    }
    return 0;
}

/*--------[ END OF FILE config.c ]-------------------------------------------*/
