/*
 * FILE: tools.c
 * 
 * ABSTRACT: Utilities and the function which fills the browsers.
 * 
 * $Id: tools.c,v 1.2 1998/12/30 19:07:24 rks Exp $
 * 
 * CHANGE LOG:
 * ##  who  ddMmmyy  defect  description
 * ==  ===  =======  ======  =================================================
 *  0  rks  13Jun98  -       initial
 */ 


#include <stdlib.h>
#include <stdarg.h>
#include <forms.h>

#include "mask.h"
#include "globals.h"
#include "tools.h"
#include "callreg.h"

#include "dbgmem.h"

/*
 * FUNCTION: begin_work
 * 
 * ABSTRACT: Sets the pointer to a clock.
 * 
 * Doesn't work sometimes.
 * 
 * PARAMETER:
 * see code
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int begin_work(Window win, char *what)
{
    if(NULL == fd_main) 
        return 1;

    fl_set_cursor(win, XC_watch);
    /*
     * I never saw this onliner, but without it the cursor shape
     * is not changed. (Thats the way it works on my machine.)
     */
    if(what != NULL)
    {
        status(what);
    }

    return 0;
}

/*
 * FUNCTION: end_work
 * 
 * ABSTRACT: Resets the mouse pointer
 * 
 * PARAMETER:
 * see code
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int end_work(Window win)
{
    if(NULL == fd_main) 
        return 1;

    fl_set_cursor(win, -1);

    /* see above */
    return 0;
}

/*
 * FUNCTION: status
 * 
 * ABSTRACT: Sets the status line.
 * 
 * PARAMETER:
 * printf-style
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int status(char *fmt, ...)
{
    va_list ap;
    char    buffer[256];
    char   *ptr;

    if(NULL == fd_main)
        return 1;
    va_start(ap, fmt);
#ifdef __GNUC__
    vsnprintf(buffer, 255, fmt, ap);
#else
    vsprintf(buffer, fmt, ap);
#endif
    va_end(ap);
    ptr = buffer;

    while(*ptr)
    {
        if(*ptr == '\n') *ptr = ' ';
        ptr++;
    }

    fl_set_object_label(fd_main->status, buffer);
    fl_draw_object_label(fd_main->status);
    
    return 0;
} 


/*
 * FUNCTION: list_label
 * 
 * ABSTRACT: Sets the label of the list browser
 * 
 * PARAMETER:
 * printf-style
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int list_label(char *fmt, ...)
{
    va_list ap;
    char    buffer[256];

    va_start(ap, fmt);
#ifdef __GNUC__
    vsnprintf(buffer, 255, fmt, ap);
#else
    vsprintf(buffer, fmt, ap);
#endif
    va_end(ap);
    fl_set_object_label(fd_main->List, buffer);
    fl_draw_object_label(fd_main->List);
    
    return 0;
} 


/*
 * FUNCTION: init_browser
 * 
 * ABSTRACT: Called by mask callback functions to performe the search. 
 * 
 * Clears the browsers, calls the perl_reg function from callreg.c
 * and fills the browsers with the output.
 * 
 * PARAMETER:
 * fdmain:       pointer to main form
 * searchstring: I don't need to explaiin this, do I?
 * 
 * RETURNVALUE:
 * see code
 * 
 */ 
int init_browser(FD_main *fd_main, char *searchstring)
{
    char  **matches;
    int     i;
    char    parameter[256];
    
    status("Cleaning up & Calling Perl script ...");
    status("Cleaning up & Calling Perl script ...");

    /* clear the browsers */
    fl_clear_browser(fd_main->List);
    fl_deselect_browser(fd_main->List);
    fl_redraw_object(fd_main->List);
    fl_clear_browser(fd_main->Output);

    if(fulltext != NULL)
    {

        for(i = 0; i < match_count; i++)
        {
            Free(fulltext[i]);
        }
        Free(fulltext);
        fulltext = NULL;
        
    }

    parameter[0] = 0;
    strcat(parameter,ParamUmlaut);
    strcat(parameter," ");
    strcat(parameter,ParamCase);
    strcat(parameter," ");
    strcat(parameter,ParamRegEx);
    strcat(parameter," ");
    strcat(parameter,ParamPhonetic);
    strcat(parameter," ");
    
    match_count = perl_reg(searchstring, parameter, &matches, &fulltext);

    status("Adding %d articles", match_count);
    
    if(fd_main->pixmap)
    {
        /* We need to reset the posthandler before calling
         * fl_delete_object, because this would call
         * the posthandler which also ties to delete the pixmap..
         */
        fl_set_object_posthandler(fd_main->BG, NULL);

        fl_delete_object(fd_main->pixmap);
        fl_redraw_object(fd_main->List);
        fd_main->pixmap = NULL;
    }

    fl_freeze_form(fd_main->List->form);
    fl_deactivate_object(fd_main->List);
    for(i = 0; i < match_count; i++)
    {
        fl_add_browser_line(fd_main->List, matches[i]);
        if((i % 50) == 0)
        {
            fl_unfreeze_form(fd_main->List->form);
            status("Adding %d articles:      ", match_count);
            status("Adding %d articles: %5d", match_count, i);
            fl_freeze_form(fd_main->List->form);
        }
    }
    fl_activate_object(fd_main->List);
    fl_unfreeze_form(fd_main->List->form);
    status("Added %d articles", match_count);

    if(match_count > 0)
    {
        fl_select_browser_line(fd_main->List, 1);
        cbList(fd_main->List, 0);
    }

    list_label("Found %d articles:", match_count);
    
    for(i = 0; i < match_count; i++)
    {
        Free(matches[i]);
    }
    Free(matches);
    
    return 0;

}

/*--------[ END OF FILE tools.c ]--------------------------------------------*/
